package nim;

public class NimBoard implements NimInterface {
    private int minStones = 3;
    private int maxStones = 5;
    private int [] rowStones;
    private int total;

    public NimBoard(int min, int max){
	minStones = min;
	maxStones = max;
	rowStones = new int[max-min+1];
	total = 0;
	for (int i = min; i <= max; i++) {
	    rowStones[i-min] = i;
	    total += i;
	}
    }
    
    public GameState evaluateGame() {
	if (total == 0) return GameState.Winning;
	else if (total == 1) return GameState.Losing;
	else return GameState.Unknown ;
    }
 
    public GameState negate(GameState g) {
	if (g ==  GameState.Winning) return GameState.Losing;
	if (g ==  GameState.Losing) return GameState.Winning;
	return GameState.Unknown;
    }

    public GameState max(GameState g1, GameState g2){
	if (g1 == GameState.Unknown && g2 != GameState.Unknown) return g2;
	if (g2 == GameState.Unknown && g1 != GameState.Unknown) return g1;
	if (g1 == GameState.Winning && g2 == GameState.Losing) return g1;
	if (g1 == GameState.Losing && g2 == GameState.Winning) return g2;
	return g1;
    }

    public GameState min(GameState g1, GameState g2){
	if (g1 == GameState.Unknown && g2 != GameState.Unknown) return g2;
	if (g2 == GameState.Unknown && g1 != GameState.Unknown) return g1;
	if (g1 == GameState.Winning && g2 == GameState.Losing) return g2;
	if (g1 == GameState.Losing && g2 == GameState.Winning) return g1;
	return g1;
    }

    public int getMinStones () {return minStones;}
    public int getMaxStones () {return maxStones;}
    public int getRows(){return (maxStones - minStones + 1);}
    public int getTotal() {return total;}
    public int getBoardSize () {return maxStones;}
    public int getStones(int x) {return (rowStones[x]);}

    public void move(int row, int stones){
	if (stones > rowStones[row]) throw new RuntimeException("Attempt to remove more stones than in row");
	if (stones <= 0) return;
	rowStones[row] = rowStones[row] - stones;
	total = total - stones;
    }

    public void undo(int row, int stones){
	rowStones[row] = rowStones[row] + stones;
	total = total + stones;
    }

    public GameState bestResult() { return evaluateGame(); }
    
    public void printMove(int row, int stones){
	System.out.println("Move: Removing "+stones+" from row "+row);
	System.out.println("      Projected result for next player: "+bestResult());
    }

    public void bestMove(){
	for (int i = 0; i <= maxStones - minStones; i++){
	    if (rowStones[i] >= 1) {
		printMove(i,1);
		move(i,1);
		return;
	    }
	}
    }
}