package nim;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.UIManager;

public class NimDrawing extends JFrame {

    private static final long serialVersionUID = 1L;
    NimInterface board;
    int boardSize;
    JPanel [][] panels;
    Font font = new Font("SansSerif",Font.BOLD,20);

    boolean win = false;
    boolean lose = false;
    LinkedList<MoveChooser> lastMoves = new LinkedList<MoveChooser>();

    public void repaint(){
	for (int i = 0; i < boardSize; i++)
	    for (int j = 0; j < boardSize; j++)
		paintCell(i,j);
	super.repaint();
    }

    public NimDrawing(NimInterface b) {
	super("Nim");
	setSize(400, 400);              // dimensions
	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); // pour terminer le

	try{
	UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
	} catch (Exception r) {
	}

	board = b;
	boardSize = board.getMaxStones();

	panels = new JPanel[boardSize][boardSize];
	JPanel content = new JPanel(new GridLayout(boardSize,boardSize));
	content.setBorder(BorderFactory.createLineBorder(Color.black));
	content.setBackground(Color.GRAY);
	for (int i = 0; i < boardSize; i++)
	    for (int j = 0; j < boardSize; j++){
		JPanel bp = new JPanel(new BorderLayout());
		bp.setPreferredSize(new Dimension(50,50));
		panels[i][j] = bp;
		content.add(bp);
		paintCell(i,j);
	    }

	JPanel pp = new JPanel(new BorderLayout(10,10));
	pp.setSize(400,400);
	pp.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	pp.add(content,BorderLayout.CENTER);

	setContentPane(pp);
	pack();
	setVisible(true); // il faut le dire !	
    }

    public void paintCell(int x, int y){
	JPanel bp = panels[x][y];
	bp.removeAll();
	if (getValue(x)> y){
	    JRadioButton b = new JRadioButton();
	    b.setBackground(Color.GRAY);
	    b.setFont(font);
	    b.setPreferredSize(new Dimension(70,50));
	    if (lose) b.setBackground(Color.RED);
	    else if (win) b.setBackground(Color.GREEN);
	    else {
		MoveChooser m = new MoveChooser(this,x,y);
		b.addActionListener(m);
	    }
	    bp.add(b,BorderLayout.CENTER);
	}
	else {	     
	    JLabel l = new JLabel("");
	    l.setFont(font);
	    bp.setBackground(Color.GRAY);
	    bp.add(l,BorderLayout.CENTER);}
	bp.revalidate();
    }

       
    public int getValue(int x) {
	if (x + 1 < board.getMinStones()) return 0;
	else return (board.getStones(x + 1 - board.getMinStones()));
    }


    public void makeMove(int x, int y){
	int row = x+1-board.getMinStones();
	int s = board.getStones(row) - y;
	System.out.println("User's Turn:");
	System.out.println("------------");
	board.printMove(row,s);
	board.move(row,s);
	if (board.getTotal() == 1) {
	    win = true;
	    System.out.println("You win!");
	    repaint();
	}
	else {
	    System.out.println("Computer's Turn:");
	    System.out.println("----------------");
	    board.bestMove();
	    if (board.getTotal() == 1) {
		lose = true;
		System.out.println("You lose!");
	    }
	    repaint();
	}
    }

    
    public void undoMove(){
	if (!lastMoves.isEmpty()) {
	    MoveChooser m = lastMoves.removeFirst();
	    board.undo(m.x,m.y);
	    System.out.println("User's Undo: Restoring "+m.y+" to row "+m.x);
	    repaint();
	}
    }

    public void doSolve(){}

    
}



