package sudoku;

import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.LinkedList;

public class SudokuBoard implements SudokuInterface {

    private GameState g = GameState.Unknown;
    private int boardSize = 9;
    private boolean regions = false;
    private int regionX = 3;
    private int regionY = 3;
    public int [][] board;
    private LinkedList<Integer> allMoves = new LinkedList<Integer>();

    public GameState getGameState() {return g;}
    public void setGameState(GameState gs) {g = gs;}
    public int getBoardSize() {return boardSize;}
    public int getValue(int x, int y) {
	return board[x][y];
    }
    public boolean hasValue(int x, int y){return board[x][y] > 0;}
    public int getRegionRows() {return regionX;}
    public int getRegionColumns() {return regionY;}
    public boolean hasRegions() {return regions;}
    public int getRegion(int x, int y){return 0;}
    public LinkedList<Integer> getAllMoves(){return allMoves;}

    public boolean isValidMove(int x, int y, int n) {
	if (n > 0 && n <= boardSize) return true;
	else return false;
    }

    public void move(int x, int y, int n) {
	if (!isValidMove(x,y,n)) 
	    System.out.println("Invalid value "+n+" at position ("+x+","+y+")");
	board[x][y] = n;
    }
    public void undo(int x, int y) {
	board[x][y] = 0;
	g = GameState.Unknown;
    }

    public LinkedList<Integer> getMoves(int x, int y) {
	LinkedList<Integer> res = new LinkedList<Integer>();
	for (int n:allMoves)
	    if (isValidMove(x,y,n)) res.add(n);
	return res;
    }

    public SudokuBoard(int size) {
	boardSize = size;
	board = new int[boardSize][boardSize];
	for (int i = 0; i < boardSize; i++) allMoves.add(i+1);
    }

    public SudokuBoard(int size, int qx, int qy) {
	boardSize = size;
	board = new int[boardSize][boardSize];
	for (int i = 0; i < boardSize; i++) allMoves.add(i+1);
	regions = true;
	regionX = qx;
	regionY = qy;
	if (boardSize % qx != 0) throw new RuntimeException("Region width does not divide boardsize");
	if (boardSize % qy != 0) throw new RuntimeException("Region height does not divide boardsize");
	if ((boardSize / qx) * (boardSize / qy) != boardSize) throw new RuntimeException("No of regions does not equal boardsize");
    }
    
    public boolean solve(){return true;}

    public void loadBoard(String file) {
	try{
	    System.out.println("Loading Sudoku puzzle from file "+file);
	    Reader reader = new FileReader(file);
	    int i = 0;
	    int j = 0;
	    try{
	    while (i < boardSize){
		while (j < boardSize){
		    char c = (char) reader.read();
		    if (c == -1) throw new RuntimeException("Reached end of file before puzzle is complete");
		    if (c == '.' || c == '0') j++;
		    else 
			if (Character.isDigit(c)) {
			    int n = Integer.parseInt(Character.toString(c));
			    if (n > 0 && n <= boardSize) move(i,j,n);
			    else throw new RuntimeException("Value larger than board size");					     
			    j++;
			}
		    
		    
		}
		i++; j = 0;
	    }
	    } catch (IOException e) {
		System.out.println("ERROR: Malformed puzzle in file "+file+": "+e.getMessage());
	    } catch (RuntimeException e) {
		System.out.println("ERROR: Malformed puzzle in file "+file+": "+e.getMessage());
	    }
	} catch (IOException e) {
	    throw new RuntimeException("ERROR: Puzze file "+file+" not found");
	}
	
    }
    
    
}