package sudoku;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.Border;

import sudoku.GameState;
import sudoku.SudokuInterface;


public class SudokuCell extends JPanel {
  private static final long serialVersionUID = 1L;
  static Border blackBorder = BorderFactory.createLineBorder(Color.black, 1);
  static Border emptyBorder = BorderFactory.createEmptyBorder();
  static Font font = new Font("SansSerif", Font.BOLD, 20);
  SudokuDrawing parent;
  int x, y;
  SudokuInterface board;
  JLabel label;
  MoveChooser chooser;

  public SudokuCell(SudokuDrawing parent, int x, int y, SudokuInterface b) {
    super(new BorderLayout());
    board = b;
    this.x = x;
    this.y = y;
    this.parent = parent;
    label = new JLabel();
    label.setHorizontalAlignment(JLabel.CENTER);
    label.setFont(font);
    label.setBorder(emptyBorder);
    chooser = new MoveChooser(parent, x, y);
    setPreferredSize(new Dimension(70, 50));
    setBorder(blackBorder);
  }

  public void refresh() {
    removeAll();
    if (board.hasValue(x, y)) {
      setBackground(Color.LIGHT_GRAY);
      label.setText(board.getValue(x, y) + "");
      add(label);
    } else {
      LinkedList<Integer> options = board.getMoves(x, y);
      if (options.size() == 0 || board.getGameState() == GameState.Losing) {
        setBackground(Color.RED);
        label.setText("X");
        add(label);
      } else {
        setBackground(Color.ORANGE);
        chooser.setOptions(options);
        add(chooser, BorderLayout.CENTER);
      }
    }
    revalidate();
  }

}
