package sudoku;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.border.Border;

import sudoku.GameState;
import sudoku.SudokuInterface;


public class SudokuDrawing extends JFrame {
  private static final long serialVersionUID = 1L;
  static Border blackBorder = BorderFactory.createLineBorder(Color.black, 1);
  static Font font = new Font("SansSerif", Font.BOLD, 20);
  SudokuInterface board;
  int boardSize;
  JPanel content;
  SudokuCell[][] panels;
  LinkedList<MoveChooser> lastMoves = new LinkedList<MoveChooser>();

  public SudokuDrawing(SudokuInterface b) {
    super("Sudoku");
    setSize(700, 500); // dimensions
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); // pour terminer le
    // programme
    try {
      UIManager
          .setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
    } catch (Exception r) {
    }
    board = b;
    boardSize = b.getBoardSize();
    panels = new SudokuCell[boardSize][boardSize];
    for (int i = 0; i < boardSize; i++)
      for (int j = 0; j < boardSize; j++) {
        panels[i][j] = new SudokuCell(this, i, j, board);
      }
    content = new JPanel();
    if (board.hasRegions())
      initRegions();
    else
      initPlain();

    JPanel pp = new JPanel(new BorderLayout(10, 10));
    pp.setSize(600, 500);
    pp.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
    pp.add(content, BorderLayout.CENTER);

    JPanel pbut = new JPanel();
    JButton b1 = new JButton("Undo");
    b1.setFont(font);
    b1.setForeground(Color.RED);
    b1.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        undoMove();
      }
    });
    pbut.add(b1);
    JButton b2 = new JButton("Solve");
    b2.setFont(font);
    b2.setForeground(Color.BLUE);
    b2.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        doSolve();
      }
    });
    pbut.add(b2);
    pp.add(pbut, BorderLayout.SOUTH);
    setContentPane(pp);
    pack();

    JMenuBar menuBar = new JMenuBar();
    JMenu menu = new JMenu("File");
    menuBar.add(menu);
    JMenuItem quit = new JMenuItem("Quit");
    menu.add(quit);
    quit.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.exit(0);
      }
    });
    setVisible(true); // il faut le dire !
    refresh();
  }

  public void initPlain() {
    content.setLayout(new GridLayout(boardSize, boardSize));
    content.setBorder(blackBorder);
    for (int i = 0; i < boardSize; i++) {
      for (int j = 0; j < boardSize; j++) {
        content.add(panels[i][j]);
      }
    }
  }

  public void initRegions() {
    int qx = board.getRegionColumns();
    int qy = board.getRegionRows();
    int xsize = boardSize / qx;
    int ysize = boardSize / qy;
    content.setLayout(new GridLayout(xsize, ysize));
    content.setBorder(blackBorder);
    for (int qi = 0; qi < xsize; qi++) {
      for (int qj = 0; qj < ysize; qj++) {
        JPanel qbp = new JPanel(new GridLayout(xsize, ysize));
        qbp.setBorder(blackBorder);
        for (int i = 0; i < qx; i++) {
          for (int j = 0; j < qy; j++) {
            int x = qi * qx + i;
            int y = qj * qy + j;
            // paintCell(x, y);
            qbp.add(panels[x][y]);
          }
        }
        content.add(qbp);
      }
    }
  }

  public void refresh() {
    for (int i = 0; i < boardSize; i++)
      for (int j = 0; j < boardSize; j++)
        panels[i][j].refresh();
  }

  public void makeMove(int x, int y, int n) {
    board.move(x, y, n);
    System.out.println("Setting board [" + x + "," + y + "] = "
        + board.getValue(x, y));
    refresh();
  }

  public void undoMove() {
    if (!lastMoves.isEmpty()) {
      MoveChooser m = lastMoves.removeFirst();
      board.undo(m.x, m.y);
      System.out.println("Resetting board [" + m.x + "," + m.y + "] = "
          + board.getValue(m.x, m.y));
      refresh();
    }
  }

  public void doSolve() {
    board.solve();
    if (board.getGameState() == GameState.Winning)
      lastMoves.clear();
    refresh();
  }

}
